class SwitchPair:

    def __init__(self):
        self.paths_dict = {}
        self.ref_hc = 30.0  # 30 Hops; Reference Hop Count

    def __str__(self):
        return str(self.paths_dict)

    def importAllPaths(self, all_paths):
        # all_paths is a list containing lists.
        # Example: [[303, 204, 313], [303, 209, 313]]

        for path in all_paths:
            x = tuple(path)
            metric = len(x) / self.ref_hc
            self.addPath(x, metric)

        return self.paths_dict

    def addPath(self, path, metric=1):
        # path is a tuple of switch dpid-s
        # Using default metric=1 to discourage immediate use of new path
        # To be used when a new link is up (and a new path exists)
        self.paths_dict[path] = metric
        reverse_path = path[::-1]
        self.paths_dict[reverse_path] = metric

    def removePath(self, path):
        # path is a tuple of switch dpid-s
        # To be used when a link in the path is down
        del self.paths_dict[path]
        reverse_path = path[::-1]
        del self.paths_dict[reverse_path]

    def updateSingleMetric(self, path, metric):
        # path is a tuple of switch dpid-s
        self.paths_dict[path] = metric

    def getAllPaths(self):
        return self.paths_dict

    def getBestPath(self, sw_a, sw_z):
        best_path = None
        best_metric = 9223372036854775807  # Chose a high value
        # The lower the metric the better. Values in range [0, 1]
        for path, metric in self.paths_dict.items():
            if path[0] == sw_a and path[-1] == sw_z and metric < best_metric:
                best_metric = metric
                best_path = path
            else:
                continue
        # print best_path, best_metric
        return (best_path, best_metric)


# The following block will only run when script is run (not when imported)
if __name__ == "__main__":
    pair = SwitchPair()

    paths = [[303, 204, 101, 201, 302, 206, 310, 205, 314, 210, 102, 209, 313],
             [303, 204, 101, 201, 302, 301, 209, 102, 206, 310, 205, 304, 210, 314, 313],
             [303, 204, 101, 201, 302, 301, 209, 102, 206, 310, 205, 314, 313],
             [303, 204, 101, 201, 302, 301, 209, 102, 210, 304, 205, 314, 313],
             [303, 204, 101, 201, 302, 301, 209, 102, 210, 314, 313],
             [303, 204, 101, 201, 302, 301, 209, 313]]

    pair.importAllPaths(paths)
    print pair

    pair.removePath((303, 204, 101, 201, 302, 301, 209, 313))
    print pair

    pair.updateSingleMetric((303, 204, 101, 201, 302, 301, 209, 102, 206, 310, 205, 304, 210, 314, 313), 0.3)
    pair.updateSingleMetric((313, 314, 205, 304, 210, 102, 209, 301, 302, 201, 101, 204, 303), 0.7)
    print pair
    print 'Best path', (303, 313), pair.getBestPath(303, 313)
    print 'Best path', (313, 303), pair.getBestPath(313, 303)
